package com.sosopay.util;

import org.apache.http.HttpEntity;
import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;

/* *
 *类名：HttpProtocolHandler
 *功能：HttpClient方式访问
 *详细：获取远程HTTP数据
 *版本：3.3
 *日期：2012-08-17
 *说明：
 *以下代码只是为了方便商户测试而提供的样例代码，商户可以根据自己网站的需要，按照技术文档编写,并非一定要使用该代码。
 *该代码仅供学习和研究支付宝接口使用，只是提供一个参考。
 */

public class HttpProtocolHandler { 

    private static String              DEFAULT_CHARSET                     = "UTF-8";

    /** 连接超时时间，由bean factory设置，缺省为8秒钟 */
    private int                        defaultConnectionTimeout            = 8000;

    /** 回应超时时间, 由bean factory设置，缺省为60秒钟 */
    private int                        defaultSoTimeout                    = 60000;

    /** 闲置连接超时时间, 由bean factory设置，缺省为60秒钟 */
    private int                        defaultIdleConnTimeout              = 60000;

    private int                        defaultMaxConnPerHost               = 30;

    private int                        defaultMaxTotalConn                 = 80;

    /** 默认等待HttpConnectionManager返回连接超时（只有在达到最大连接数时起作用）：1秒*/
    private static final long          defaultHttpConnectionManagerTimeout = 3 * 1000;

    /**
     * HTTP连接管理器，该连接管理器必须是线程安全的.
     */
    private PoolingHttpClientConnectionManager  connectionManager;

    private static HttpProtocolHandler httpProtocolHandler  = new HttpProtocolHandler();

    /**
     * 工厂方法
     * 
     * @return
     */
    public static HttpProtocolHandler getInstance() {
        return httpProtocolHandler;
    }

    /**
     * 私有的构造方法
     */
    private HttpProtocolHandler() {
        // 创建一个线程安全的HTTP连接池
    	connectionManager= new PoolingHttpClientConnectionManager();
        // 将最大连接数增加到200
    	connectionManager.setMaxTotal(defaultMaxTotalConn);
        // 将每个路由基础的连接增加到20
    	connectionManager.setDefaultMaxPerRoute(defaultMaxConnPerHost);
//    	ConnectionConfig config=ConnectionConfig.custom().setCharset(Charsets.UTF_8).build();  
//    	connectionManager.setDefaultConnectionConfig(config);
    	//将目标主机的最大连接数增加到50
//    	HttpHost localhost = new HttpHost("www.yeetrack.com", 80);
//    	cm.setMaxPerRoute(new HttpRoute(localhost), 50);
    }

    /**
     * 执行Http请求
     * 金融
     * @param request 请求数据
     * @param strParaFileName 文件类型的参数名
     * @param strFilePath 文件路径
     * @return 
     * @throws HttpException, IOException 
     */
    public String execute(HttpUriRequest request) throws Exception {
        RequestConfig requestConfig = RequestConfig.custom()
				.setSocketTimeout(defaultSoTimeout)
				.setConnectTimeout(defaultConnectionTimeout).build();// 设置请求和传输超时时间
    	//通过更复杂的PoolingHttpClientConnectionManager来管理多个连接，适合多线程中的请求
        CloseableHttpClient httpclient = HttpClients.custom()
        		.setConnectionManager(connectionManager).setDefaultRequestConfig(requestConfig)
        		.build();
        CloseableHttpResponse httpResponse=httpclient.execute(request);
        //请求返回结果
     	String strResult = EntityUtils.toString(httpResponse.getEntity(),DEFAULT_CHARSET);
        httpResponse.close();
        return strResult;
    }
    
    @SuppressWarnings("resource")
  	public String executeHttps(HttpUriRequest request){
      	try {
  			HttpClient httpClient = new SSLClient();
  			
  			HttpResponse response = httpClient.execute(request);  
  			String responseString = "";
  	        if(response != null){  
  	            HttpEntity resEntity = response.getEntity();  
  	            if(resEntity != null){  
  	           	 responseString = EntityUtils.toString(resEntity,"UTF-8"); 
  	            }  
  	        } 
  	        return responseString;
  		} catch (Exception e) {
  			// TODO Auto-generated catch block
  			e.printStackTrace();
  			return "";
  		} 
      }

}
